%%
% This code accompanies the paper:
%
% "Localized Manifold Harmonics for Spectral Shape Analysis"
% S. Melzi, E. Rodola, U. Castellani, M. Bronstein
% Computer Graphics Forum 2017
%
% Please cite the paper above if you use this code.
%
% Written by S. Melzi and E. Rodola, 2017
%
%%
addpath('./tools/')
addpath('./method/')

clear all
close all
clc

%% Load parameters, shape and region

params = struct;
params.k_global = 20; % no. of standard LB eigenfunctions
params.k_local = 10;  % no. of localized manifold harmonics
params.mu = 5e1;      % locality
params.eta = 1e2;     % orthogonality

M = load_off('./dog2.off');

% hard and soft regions are allowed, try examples below

load('./regions.mat')
region = regions(:,1); % hard region
% region = regions(:,2); % soft region

figure, colormap(bluewhitered)
plot_scalar_map(M, region), colorbar
title('Selected region')

%% Compute global and localized manifold harmonics

[M.Phi, M.Lambda, M.A, M.W] = calc_LB(M, params.k_global);

[Psi, Gamma] = compute_lmh(M, region, params);

%% Plot

figure, colormap(bluewhitered)
for i=1:10
    subplot(2,5,i)
    plot_scalar_map(M, Psi(:,i))
    title(sprintf('%d', i))
    v = max(abs(Psi(:,i)));
    caxis([-v v])
end

figure, colormap(bluewhitered)
imagesc([M.Phi Psi]'*M.A*[M.Phi Psi])
axis image; colorbar
title('Orthogonality (should approximate identity)')

figure
plot(1:params.k_global, M.Lambda, 'LineWidth', 2), hold on
plot(params.k_global+1:params.k_global+params.k_local, Gamma, 'LineWidth', 2)
plot(params.k_global:params.k_global+1, [M.Lambda(end) Gamma(1)], 'k:', 'LineWidth', 2)
legend('LB eigenvalues', 'LMH eigenvalues', 'Spectral gap', 'Location', 'NorthWest')
set(gca, 'xlim', [1 params.k_global+params.k_local])
ylabel('Eigenvalue')
xlabel('Index')
title('Spectra')
